/*! Tripetto Collector 1.7.1-50 - Copyright (C) 2020 Tripetto B.V. - All Rights Reserved */

declare module "@marblecore/metrics" {
    /** Explicit exports */
    export { IPoint } from "@marblecore/metrics/lib/point";
    export { IRectangleProperties } from "@marblecore/metrics/lib/properties";
    export { Rectangle } from "@marblecore/metrics/lib/rectangle";
    export { Rectangles } from "@marblecore/metrics/lib/rectangles";
    export { Coordinate } from "@marblecore/metrics/lib/coordinate";
    export { Dimensions } from "@marblecore/metrics/lib/dimensions";
    export { Spacing } from "@marblecore/metrics/lib/spacing";
}

declare module "@marblecore/metrics/lib/point" {
    export interface IPoint {
        /** Specifies the horizontal coordinate. */
        readonly x: number;
        /** Specifies the vertical coordinate. */
        readonly y: number;
    }
}

declare module "@marblecore/metrics/lib/properties" {
    export interface IRectangleProperties<T extends string> {
        /** Specifies the rectangle name. */
        name: T;
        /** Specifies the initial width of the rectangle. */
        readonly width?: number;
        /** Specifies the initial height of the rectangle. */
        readonly height?: number;
        /** Specifies the minimal width of the rectangle. */
        readonly minWidth?: number;
        /** Specifies the minimal height of the rectangle. */
        readonly minHeight?: number;
        /** Specifies the maximal width of the rectangle. */
        readonly maxWidth?: number;
        /** Specifies the minimal width of the rectangle. */
        readonly maxHeight?: number;
        /** Specifies the left spacing of the rectangle. */
        readonly spacingLeft?: number;
        /** Specifies the top spacing of the rectangle. */
        readonly spacingTop?: number;
        /** Specifies the right spacing of the rectangle. */
        readonly spacingRight?: number;
        /** Specifies the bottom spacing of the rectangle. */
        readonly spacingBottom?: number;
        /** Specifies the left hint correction. */
        readonly hintCorrectionLeft?: number;
        /** Specifies the top hint correction. */
        readonly hintCorrectionTop?: number;
        /** Specifies the right hint correction. */
        readonly hintCorrectionRight?: number;
        /** Specifies the bottom hint correction. */
        readonly hintCorrectionBottom?: number;
        /** Specifies the horizontal mean correction. */
        readonly meanCorrectionHorizontal?: number;
        /** Specifies the vertical mean correction. */
        readonly meanCorrectionVertical?: number;
        /** Specifies the names of the rectangles which should align to this rectangle. */
        readonly align?: T[];
    }
}

declare module "@marblecore/metrics/lib/rectangle" {
    import { Coordinate } from "@marblecore/metrics/lib/coordinate";
    import { Dimensions } from "@marblecore/metrics/lib/dimensions";
    export class Rectangle {
        /**
         * Constructs a new rectangle.
         * @param pProperties Specifies the rectangle properties.
         */
        constructor(pProperties?: {
            /** Specifies the left position. */
            left?: number;
            /** Specifies the top position. */
            top?: number;
            /** Specifies the right position. */
            right?: number;
            /** Specifies the bottom position. */
            bottom?: number;
            /** Specifies the width of the rectangle. If the width is supplied the specified right position is ignored. */
            width?: number;
            /** Specifies the height of the rectangle. If the height is supplied the specified bottom position is ignored. */
            height?: number;
            /** Specifies the minimum width. */
            minWidth?: number;
            /** Specifies the minimum height. */
            minHeight?: number;
            /** Specifies the maximum width. */
            maxWidth?: number;
            /** Specifies the maximum height. */
            maxHeight?: number;
        });
        /** Retrieves the dimensions. */
        readonly dimensions: Dimensions;
        /** Retrieves the coordinates. */
        readonly coordinate: Coordinate;
        /** Sets the left coordinate of the rectangle. */
        left: number;
        /** Sets the top coordinate of the rectangle. */
        top: number;
        /** Sets the right coordinate of the rectangle. */
        right: number;
        /** Sets the bottom coordinate of the rectangle. */
        bottom: number;
        /** Sets the width of the rectangle. */
        width: number;
        /** Sets the height of the rectangle. */
        height: number;
        /** Retrieves the rounded width of the rectangle.  */
        readonly roundedWidth: number;
        /** Retrieves the rounded height of the rectangle.  */
        readonly roundedHeight: number;
        /** Retrieves the unranged width of the rectangle.  */
        readonly w: number;
        /** Retrieves the unranged height of the rectangle.  */
        readonly h: number;
        /** Retrieves an immutable copy of the rectangle. */
        readonly copy: Rectangle;
        /**
         * Verifies if the supplied point is within the rectangle.
         * @param nX Specifies the horizontal position.
         * @param nY Specifies the vertical position.
         * @return Returns `true` if the point is within the rectangle.
         */
        isIn(nX?: number, nY?: number): boolean;
    }
}

declare module "@marblecore/metrics/lib/rectangles" {
    import { Coordinate } from "@marblecore/metrics/lib/coordinate";
    import { Dimensions } from "@marblecore/metrics/lib/dimensions";
    import { Rectangle } from "@marblecore/metrics/lib/rectangle";
    import { IRectangleProperties } from "@marblecore/metrics/lib/properties";
    /**
     * Implements a rectangle manager which can be used to manage rectangle relationships. The relation between those rectangles is described
     * by specifying characteristics of the rectangles, such as padding, minimal and maximum dimensions, etc. When the dimensions of a
     * rectangle change, the resulting dimensions of the surrounding rectangles are calculated.
     * @param T Specifies the rectangle names.
     */
    export class Rectangles<T extends string> {
        /**
         * Constructs a new rectangle repository.
         * @param pRectangles Specifies an array with rectangles.
         */
        constructor(pRectangles?: IRectangleProperties<T>[]);
        /** Sets the left coordinate of the comprehensive rectangle. */
        left: number;
        /** Sets the top coordinate of the comprehensive rectangle. */
        top: number;
        /** Retrieves the right coordinate of the comprehensive rectangle. */
        readonly right: number;
        /** Retrieves the bottom coordinate of the comprehensive rectangle. */
        readonly bottom: number;
        /** Retrieves the width of the comprehensive rectangle. */
        readonly width: number;
        /** Retrieves the height of the comprehensive rectangle. */
        readonly height: number;
        /** Returns the width and height of the comprehensive rectangle. */
        readonly dimensions: Dimensions;
        /**
         * Retrieves the inner left coordinate of the comprehensive rectangle. This
         * is the rectangle which includes all rectangles without spacing.
         */
        readonly innerLeft: number;
        /**
         * Retrieves the inner top coordinate of the comprehensive rectangle. This
         * is the rectangle which includes all rectangles without spacing.
         */
        readonly innerTop: number;
        /**
         * Retrieves the inner right coordinate of the comprehensive rectangle. This
         * is the rectangle which includes all rectangles without spacing.
         */
        readonly innerRight: number;
        /**
         * Retrieves the inner bottom coordinate of the comprehensive rectangle.
         * This is the rectangle which includes all rectangles without spacing.
         */
        readonly innerBottom: number;
        /**
         * Retrieves the inner width of the comprehensive rectangle. This is the
         * rectangle which includes all rectangles without spacing.
         */
        readonly innerWidth: number;
        /**
         * Retrieves the inner height of the comprehensive rectangle. This is the
         * rectangle which includes all rectangles without spacing.
         */
        readonly innerHeight: number;
        /** Sets the shift mode. */
        isShifted: boolean;
        /** Sets the left move. */
        shiftLeft: number;
        /** Sets the top shift. */
        shiftTop: number;
        /** Retrieves the right position of the shifted element. */
        readonly shiftRight: number;
        /** Retrieves the bottom position of the shifted element. */
        readonly shiftBottom: number;
        /** Retrieves the last horizontal position shift. */
        readonly shiftHorizontal: number;
        /** Retrieves the last vertical position shift. */
        readonly shiftVertical: number;
        /** Retrieves the horizontal shift mean. */
        readonly shiftMeanHorizontal: number;
        /** Retrieves the vertical shift mean. */
        readonly shiftMeanVertical: number;
        /**
         * Iterates through all rectangles.
         * @param fnRectangle Specifies the function which is called for each rectangle.
         * @param fnRectangle.pRectangle Reference to the rectangle metrics.
         * @param fnRectangle.sRectangle Contains the rectangle name.
         * @param nZoom Specifies an optional zoom factor.
         */
        each(
            fnRectangle: (pRectangle: Rectangle, sRectangle: T) => void,
            nZoom?: number
        ): void;
        /**
         * Sets or retrieves the minimum width of the specified rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nMinWidth Specifies the minimum width.
         * @return Returns the minimal width.
         */
        minWidth(sRectangle: T, nMinWidth?: number): number;
        /**
         * Sets or retrieves the minimum height of the specified rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nMinHeight Specifies the minimum height.
         * @return Returns the minimal height.
         */
        minHeight(sRectangle: T, nMinHeight?: number): number;
        /**
         * Sets or retrieves the maximum width of the specified rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nMaxWidth Specifies the maximum width.
         * @return Returns the maximum width.
         */
        maxWidth(sRectangle: T, nMaxWidth?: number): number;
        /**
         * Sets or retrieves the maximum height of the specified rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nMaxHeight Specifies the maximum height.
         * @return Returns the maximum height.
         */
        maxHeight(sRectangle: T, nMaxHeight?: number): number;
        /**
         * Retrieves the horizontal center of the specified rectangle or the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the mean of the comprehensive rectangle will be calculated.
         * @return Returns the horizontal mean.
         */
        meanHorizontal(sRectangle?: T): number;
        /**
         * Retrieves the vertical center of the specified rectangle or the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the mean of the comprehensive rectangle will be calculated.
         * @return Returns the vertical mean.
         */
        meanVertical(sRectangle?: T): number;
        /**
         * Retrieves the rectangle for the specified rectangle or the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nZoom Specifies an optional zoom factor.
         * @return Returns the rectangle.
         */
        rectangle(sRectangle?: T, nZoom?: number): Rectangle;
        /**
         * Retrieves the applied rectangle for the specified rectangle. This is the rectangle which is currently stored in the buffer.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nZoom Specifies an optional zoom factor.
         * @return Returns the rectangle.
         */
        rectangleApplied(sRectangle?: T, nZoom?: number): Rectangle;
        /**
         * Sets or retrieves the width of a rectangle or retrieves the width of the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nWidth Specifies the width.
         * @return Returns the width.
         */
        rectangleWidth(sRectangle?: T, nWidth?: number): number;
        /**
         * Sets or retrieves the height of a rectangle or retrieves the width of the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nWidth Specifies the width.
         * @return Returns the width.
         */
        rectangleHeight(sRectangle?: T, nHeight?: number): number;
        /**
         * Verifies if the specified rectangle is aligned with another rectangle.
         * @param sRectangleA Specifies the first rectangle.
         * @param sRectangleB Specifies the second rectangle.
         * @return Returns `true` if the rectangle is aligned.
         */
        isRectangleAligned(sRectangleA: T, sRectangleB: T): boolean;
        /**
         * Sets or retrieves the left spacing of a rectangle or retrieves the left comprehensive rectangle spacing.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nSpacingLeft Specifies the left spacing.
         * @return Returns the left spacing.
         */
        spacingLeft(sRectangle?: T, nSpacingLeft?: number): number;
        /**
         * Sets or retrieves the top spacing of a rectangle or retrieves the top comprehensive rectangle spacing.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nSpacingTop Specifies the top spacing.
         * @return Returns the top spacing.
         */
        spacingTop(sRectangle?: T, nSpacingTop?: number): number;
        /**
         * Sets or retrieves the right spacing of a rectangle or retrieves the right comprehensive rectangle spacing.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nSpacingLeft Specifies the right spacing.
         * @return Returns the right spacing.
         */
        spacingRight(sRectangle?: T, nSpacingRight?: number): number;
        /**
         * Sets or retrieves the bottom spacing of a rectangle or retrieves the bottom comprehensive rectangle spacing.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nSpacingBottom Specifies the bottom spacing.
         * @return Returns the bottom spacing.
         */
        spacingBottom(sRectangle?: T, nSpacingBottom?: number): number;
        /**
         * Retrieves the horizontal spacing of a rectangle or the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @return Returns the horizontal spacing.
         */
        spacingHorizontal(sRectangle?: T): number;
        /**
         * Retrieves the vertical spacing of a rectangle or the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @return Returns the vertical spacing.
         */
        spacingVertical(sRectangle?: T): number;
        /**
         * Sets or retrieves the left hint correction of a rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nHintLeft Specifies the left hint correction.
         * @return Returns the left hint correction.
         */
        hintCorrectionLeft(sRectangle: T, nHintLeft?: number): number;
        /**
         * Sets or retrieves the top hint correction of a rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nHintTop Specifies the top hint correction.
         * @return Returns the top hint correction.
         */
        hintCorrectionTop(sRectangle: T, nHintTop?: number): number;
        /**
         * Sets or retrieves the right hint correction of a rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nHintRight Specifies the right hint correction.
         * @return Returns the right hint correction.
         */
        hintCorrectionRight(sRectangle: T, nHintRight?: number): number;
        /**
         * Sets or retrieves the bottom hint correction of a rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nHintBottom Specifies the bottom hint correction.
         * @return Returns the bottom hint correction.
         */
        hintCorrectionBottom(sRectangle: T, nHintBottom?: number): number;
        /**
         * Retrieves the shifted rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @return Returns the rectangle.
         */
        shiftRectangle(sRectangle?: T): Rectangle;
        /**
         * Positions the rectangles.
         * @return Returns `true` if the position is changed.
         */
        position(): boolean;
        /**
         * Resizes the rectangles.
         * @return Returns which properties are changed (`none`, `width`, `height` or `both`).
         */
        resize(): "none" | "width" | "height" | "both";
        /**
         * Verifies if the supplied coordinate is within one or more rectangles.
         * @param nX Specifies the left position.
         * @param nY Specifies the top position.
         * @param bSpacing Specifies if the spacing needs to be applied.
         * @return Returns `true` if the coordinate is within one or more rectangles.
         */
        isCoordinateIn(
            nX: number | undefined,
            nY: number | undefined,
            bSpacing?: boolean
        ): boolean;
        /**
         * Verifies if the supplied coordinate is within the comprehensive rectangle.
         * @param nX Specifies the left position.
         * @param nY Specifies the top position.
         * @param bInner Specifies if the inner rectangle needs to be verified.
         * @param pCorrection Specifies correction coordinates.
         * @return Returns `true` if the coordinate is within the comprehensive rectangle.
         */
        isCoordinateInRectangles(
            nX: number | undefined,
            nY: number | undefined,
            bInner?: boolean,
            pCorrection?: Coordinate
        ): boolean;
        /**
         * Verifies if the supplied coordinate is within the specified rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nX Specifies the left position.
         * @param nY Specifies the top position.
         * @param bSpacing Specifies if spacing needs to be included.
         * @return Returns `true` if the coordinate is within the specified rectangle.
         */
        isCoordinateInRectangle(
            sRectangle: T,
            nX: number | undefined,
            nY: number | undefined,
            bSpacing?: boolean
        ): boolean;
    }
}

declare module "@marblecore/metrics/lib/coordinate" {
    export class Coordinate {
        /** Specifies the left coordinate. */
        left: number;
        /** Specifies the top coordinate. */
        top: number;
        /** Specifies the right coordinate. */
        right: number;
        /** Specifies the bottom coordinate. */
        bottom: number;
        /**
         * Constructs a new coordinate.
         * @param nLeft Specifies the left coordinate.
         * @param nTop Specifies the top coordinate.
         * @param nRight Specifies the right coordinate.
         * @param nBottom Specifies the bottom coordinate.
         */
        constructor(
            nLeft?: number,
            nTop?: number,
            nRight?: number,
            nBottom?: number
        );
    }
}

declare module "@marblecore/metrics/lib/dimensions" {
    export class Dimensions {
        /** Retrieves the `none` constant for the width and height. */
        static readonly NONE: number;
        /** Contains the current width. */
        m_nWidth: number;
        /** Contains the current height. */
        m_nHeight: number;
        /** Contains the minimum width. */
        m_nMinWidth: number;
        /** Contains the minimum height. */
        m_nMinHeight: number;
        /** Contains the maximum width. */
        m_nMaxWidth: number;
        /** Contains the maximum height. */
        m_nMaxHeight: number;
        /**
         * Constructs a new dimension object.
         * @param nWidth Specifies the width.
         * @param nHeight Specifies the height.
         * @param nMinWidth Specifies the minimum width.
         * @param nMinHeight Specifies the minimum height.
         * @param nMaxWidth Specifies the maximum width.
         * @param nMaxHeight Specifies the minimum height.
         */
        constructor(
            nWidth?: number,
            nHeight?: number,
            nMinWidth?: number,
            nMinHeight?: number,
            nMaxWidth?: number,
            nMaxHeight?: number
        );
        /** Sets the minimum width. */
        minWidth: number;
        /** Sets the minimum height. */
        minHeight: number;
        /** Sets the maximum width. */
        maxWidth: number;
        /** Sets the maximum height. */
        maxHeight: number;
        /** Sets the current width. */
        width: number;
        /** Sets the current height. */
        height: number;
        /** Retrieves if the width is specified. */
        readonly isWidthSpecified: boolean;
        /** Retrieves if the height is specified. */
        readonly isHeightSpecified: boolean;
        /** Retrieves the unranged width. */
        readonly w: number;
        /** Retrieves the unranged height. */
        readonly h: number;
    }
}

declare module "@marblecore/metrics/lib/spacing" {
    /** Dependencies */
    import { Coordinate } from "@marblecore/metrics/lib/coordinate";
    export class Spacing extends Coordinate {
        /**
         * Constructs a new spacing container.
         * @param nLeft Specifies the left spacing.
         * @param nTop Specifies the top spacing.
         * @param nRight Specifies the right spacing.
         * @param nBottom Specifies the bottom spacing.
         */
        constructor(
            nLeft?: number,
            nTop?: number,
            nRight?: number,
            nBottom?: number
        );
        /** Retrieves the total horizontal spacing. */
        readonly horizontal: number;
        /** Retrieves the total vertical spacing. */
        readonly vertical: number;
    }
}
